import { JupyterFrontEndPlugin } from "@jupyterlab/application";
/**
 * Initialization data for the jupyterlab-jupytext extension.
 */
declare const extension: JupyterFrontEndPlugin<void>;
export default extension;
